﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Clients.Hive.Views {
  [View("HiveJob View")]
  [Content(typeof(HiveJob), true)]
  [Content(typeof(HiveJob<>), false)]
  public partial class HiveJobView : ItemView {
    public new HiveJob Content {
      get { return (HiveJob)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    public HiveJobView() {
      InitializeComponent();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.JobItemChanged += new EventHandler(Content_JobItemChanged);
      Content.JobChanged += new EventHandler(Content_JobChanged);
      Content.JobStateChanged += new EventHandler(Content_JobStateChanged);
    }

    protected override void DeregisterContentEvents() {
      Content.JobItemChanged -= new EventHandler(Content_JobItemChanged);
      Content.JobChanged -= new EventHandler(Content_JobChanged);
      Content.JobStateChanged -= new EventHandler(Content_JobStateChanged);
      base.DeregisterContentEvents();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null && Content.Job != null) {
        computeInParallelCheckBox.Checked = Content.JobItem.ComputeInParallel;
      } else {
        computeInParallelCheckBox.Checked = false;
      }
      Content_JobItemChanged(this, EventArgs.Empty);
      Content_JobChanged(this, EventArgs.Empty);
      Content_JobStateChanged(this, EventArgs.Empty);
    }

    protected virtual void RegisterJobEvents() {
      if (Content != null && Content.Job != null) {
        Content.JobItem.ComputeInParallelChanged += new EventHandler(OptimizerJob_ComputeInParallelChanged);
        Content.JobItem.ItemChanged += new EventHandler(Job_ItemChanged);
      }
    }

    protected virtual void DeregisterJobEvents() {
      if (Content != null && Content.Job != null) {
        Content.JobItem.ComputeInParallelChanged -= new EventHandler(OptimizerJob_ComputeInParallelChanged);
        Content.JobItem.ItemChanged -= new EventHandler(Job_ItemChanged);
      }
    }

    #region Content Events
    protected virtual void Content_JobChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobChanged), sender, e);
      } else {
        if (Content != null && Content.Job != null) {
          this.jobIdTextBox.Text = Content.Job.Id.ToString();
          this.dateCreatedTextBox.Text = Content.Job.DateCreated.HasValue ? Content.Job.DateCreated.ToString() : string.Empty;
          this.priorityTextBox.Text = Content.Job.Priority.ToString();
          this.coresNeededTextBox.Text = Content.Job.CoresNeeded.ToString();
          this.memoryNeededTextBox.Text = Content.Job.MemoryNeeded.ToString();
        } else {
          this.jobIdTextBox.Text = string.Empty;
          this.dateCreatedTextBox.Text = string.Empty;
          this.priorityTextBox.Text = string.Empty;
          this.coresNeededTextBox.Text = string.Empty;
          this.memoryNeededTextBox.Text = string.Empty;
        }
      }
    }

    protected virtual void Content_JobItemChanged(object sender, EventArgs e) {
      RegisterJobEvents();
      Job_ItemChanged(this, e);
    }

    protected virtual void Job_ItemChanged(object sender, EventArgs e) {
      if (Content != null && Content.Job != null && Content.JobItem.Item != null) {
        optimizerItemView.Content = Content.JobItem.Item;
      } else {
        optimizerItemView.Content = null;
      }
    }

    protected virtual void Content_JobStateChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobStateChanged), sender, e);
      } else {
        if (Content != null && Content.Job != null) {
          this.stateTextBox.Text = Content.Job.State.ToString();
          this.executionTimeTextBox.Text = Content.Job.ExecutionTime.ToString();
          this.dateFinishedTextBox.Text = Content.Job.DateFinished.ToString();
          this.exceptionTextBox.Text = Content.Job.CurrentStateLog != null ? Content.Job.CurrentStateLog.Exception : string.Empty;
          this.lastUpdatedTextBox.Text = Content.Job.LastJobDataUpdate.ToString();
          if (Content.JobItem.ComputeInParallel) {
            this.stateLogViewHost.Content = new StateLogListList(
                this.Content.ChildHiveJobs.Select(child => new StateLogList(child.Job.StateLog)
              ));
          } else {
            this.stateLogViewHost.Content = new StateLogList(Content.Job.StateLog);
          }
        } else {
          this.stateTextBox.Text = string.Empty;
          this.executionTimeTextBox.Text = string.Empty;
          this.dateCalculatedText.Text = string.Empty;
          this.dateFinishedTextBox.Text = string.Empty;
          this.exceptionTextBox.Text = string.Empty;
          this.stateLogViewHost.Content = null;
          this.lastUpdatedTextBox.Text = string.Empty;
        }
        SetEnabledStateOfControls();
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      this.jobIdTextBox.ReadOnly = true;
      this.stateTextBox.ReadOnly = true;
      this.executionTimeTextBox.ReadOnly = true;
      this.dateCreatedTextBox.ReadOnly = true;
      this.dateCalculatedText.ReadOnly = true;
      this.dateFinishedTextBox.ReadOnly = true;
      this.exceptionTextBox.ReadOnly = true;
      this.lastUpdatedTextBox.ReadOnly = true;

      this.priorityTextBox.ReadOnly = this.ReadOnly;
      this.coresNeededTextBox.ReadOnly = this.ReadOnly;
      this.memoryNeededTextBox.ReadOnly = this.ReadOnly;
      this.computeInParallelCheckBox.Enabled = !this.ReadOnly && this.Content != null && this.Content.JobItem != null && this.Content.JobItem.IsParallelizable;

      this.modifyItemButton.Enabled = (Content != null && Content.JobItem.Item != null && (Content.Job.State == JobState.Paused || Content.Job.State == JobState.Offline || Content.Job.State == JobState.Finished || Content.Job.State == JobState.Failed || Content.Job.State == JobState.Aborted));

      optimizerItemView.ReadOnly = true;
    }

    protected virtual void OptimizerJob_ComputeInParallelChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(OptimizerJob_ComputeInParallelChanged), sender, e);
      } else {
        computeInParallelCheckBox.Checked = Content.JobItem.ComputeInParallel;
      }
    }
    #endregion

    #region Child Control Events
     protected virtual void computeInParallelCheckBox_CheckedChanged(object sender, EventArgs e) {
      if (Content != null && Content.JobItem != null) {
        this.Content.JobItem.ComputeInParallel = this.computeInParallelCheckBox.Checked;
      }
    }

    protected virtual void exceptionTextBox_DoubleClick(object sender, EventArgs e) {
      using (TextDialog dialog = new TextDialog("Exception", exceptionTextBox.Text, ReadOnly || !Content.CanChangeDescription)) {
        if (dialog.ShowDialog(this) == DialogResult.OK)
          Content.Description = dialog.Content;
      }
    }

    protected virtual void modifyItemButton_Click(object sender, EventArgs e) {
      MainFormManager.MainForm.ShowContent(Content.JobItem.Item);
    }
    #endregion
  }
}
