﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Hive;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Clients.Hive.Jobs {
  [Item("Optimizer Job", "Represents Job which executes a IOptimizer object.")]
  [StorableClass]
  public class OptimizerJob : ItemJob {
    public override bool IsParallelizable {
      get { return this.Item is Experiment || this.Item is BatchRun; }
    }

    public new IOptimizer Item {
      get { return (IOptimizer)base.Item; }
      set { base.Item = value; }
    }

    [Storable]
    private int indexInParentOptimizerList = -1;
    public int IndexInParentOptimizerList {
      get { return indexInParentOptimizerList; }
      set { this.indexInParentOptimizerList = value; }
    }

    public OptimizerJob() : base() { }
    public OptimizerJob(IOptimizer optimizer)
      : this() {
        this.Item = optimizer;

      if (optimizer is Experiment) {
        this.ComputeInParallel = true;
      } else if (optimizer is BatchRun) {
        this.ComputeInParallel = false;
      } else {
        this.ComputeInParallel = false;
      }
    }
    [StorableConstructor]
    protected OptimizerJob(bool deserializing) { }
    protected OptimizerJob(OptimizerJob original, Cloner cloner)
      : base(original, cloner) {
      this.IndexInParentOptimizerList = original.IndexInParentOptimizerList;
      this.CollectChildJobs = original.CollectChildJobs;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new OptimizerJob(this, cloner);
    }

    /// <summary>
    /// Casts the Optimizer to an Experiment. Returns null if cast was not successfull.
    /// </summary>
    public Experiment OptimizerAsExperiment {
      get { return Item as Experiment; }
    }

    /// <summary>
    /// Casts the Optimizer to an BatchRun. Returns null if cast was not successfull.
    /// </summary>
    public BatchRun OptimizerAsBatchRun {
      get { return Item as BatchRun; }
    }

    #region IJob Members

    public override ExecutionState ExecutionState {
      get { return Item.ExecutionState; }
    }

    public override TimeSpan ExecutionTime {
      get { return Item.ExecutionTime; }
    }

    public override void Prepare() {
      Item.Prepare();
    }

    public override void Start() {
      if ((Item is Experiment && OptimizerAsExperiment.Optimizers.Count == 0) || // experiment would not fire OnStopped if it has 0 optimizers
          (Item is BatchRun && OptimizerAsBatchRun.Optimizer == null)) { // batchrun would not fire OnStopped if algorithm == null
        OnJobStopped();
      } else {
        Item.Start();
      }
    }

    public override void Pause() {
      Item.Pause();
    }

    public override void Stop() {
      Item.Stop();
    }

    public override void Resume(IEnumerable<IJob> childJobs) {
      OnJobStopped();
    }
    #endregion

    #region Optimizer Events
    protected override void RegisterItemEvents() {
      base.RegisterItemEvents();
      Item.Stopped += new EventHandler(optimizer_Stopped);
      Item.Paused += new EventHandler(optimizer_Paused);
      Item.ExceptionOccurred += new EventHandler<EventArgs<Exception>>(optimizer_ExceptionOccurred);
      Item.DescriptionChanged += new EventHandler(optimizer_DescriptionChanged);
      Item.NameChanged += new EventHandler(optimizer_NameChanged);
      Item.NameChanging += new EventHandler<CancelEventArgs<string>>(optimizer_NameChanging);
    }

    protected virtual void DeregisterOptimizerEvents() {
      Item.Stopped -= new EventHandler(optimizer_Stopped);
      Item.Paused -= new EventHandler(optimizer_Paused);
      Item.ExceptionOccurred -= new EventHandler<EventArgs<Exception>>(optimizer_ExceptionOccurred);
      Item.DescriptionChanged -= new EventHandler(optimizer_DescriptionChanged);
      Item.NameChanged -= new EventHandler(optimizer_NameChanged);
      Item.NameChanging -= new EventHandler<CancelEventArgs<string>>(optimizer_NameChanging);
      base.DeregisterItemEvents();
    }

    protected void optimizer_NameChanging(object sender, CancelEventArgs<string> e) {
      this.OnNameChanging(e.Value, e.Cancel);
    }

    protected void optimizer_NameChanged(object sender, EventArgs e) {
      this.OnNameChanged();
    }

    protected void optimizer_DescriptionChanged(object sender, EventArgs e) {
      this.OnDescriptionChanged();
    }

    protected virtual void optimizer_ExceptionOccurred(object sender, EventArgs<Exception> e) {
      OnJobFailed(e);
    }

    protected virtual void optimizer_Stopped(object sender, EventArgs e) {
      OnJobStopped();
    }

    protected virtual void optimizer_Paused(object sender, EventArgs e) {
      OnJobPaused();
    }
    #endregion

    #region INamedItem Members
    public override bool CanChangeDescription {
      get { return Item.CanChangeDescription; }
    }

    public override bool CanChangeName {
      get { return Item.CanChangeName; }
    }

    public override string Description {
      get { return Item.Description; }
      set { Item.Description = value; }
    }

    public override string Name {
      get { return Item.Name; }
      set { Item.Name = value; }
    }
    #endregion
  }
}
